# Generated by Django 5.2.1 on 2025-09-19 12:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0207_remove_redundant_indexes'),
        ('netbox_firmware', '0004_firmware_manufacturer_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='firmware',
            name='manufacturer',
            field=models.ForeignKey(default=7, on_delete=django.db.models.deletion.PROTECT, related_name='Firmware', to='dcim.manufacturer'),
        ),
    ]
