from .schema import (
    FirmwareQuery,
    FirmwareAssignmentQuery,
    BiosQuery,
    BiosAssignmentQuery,
)

schema = [
    FirmwareQuery,
    FirmwareAssignmentQuery,
    BiosQuery,
    BiosAssignmentQuery,
]