# Generated by Django 5.2.1 on 2025-09-11 11:45

import django.db.models.deletion
import django.db.models.functions.text
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0207_remove_redundant_indexes'),
        ('extras', '0128_tableconfig'),
        ('netbox_firmware', '0003_alter_firmware_options_alter_firmware_device_type_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='firmware',
            name='manufacturer',
            field=models.ForeignKey(default=6, on_delete=django.db.models.deletion.PROTECT, related_name='Firmware', to='dcim.manufacturer'),
        ),
        migrations.AlterConstraint(
            model_name='bios',
            name='netbox_firmware_bios_unique_name',
            constraint=models.UniqueConstraint(django.db.models.functions.text.Lower('name'), name='netbox_firmware_bios_unique_name', violation_error_message="The BIOS 'Name' must be unique."),
        ),
        migrations.AlterConstraint(
            model_name='firmware',
            name='netbox_firmware_firmware_unique_name',
            constraint=models.UniqueConstraint(django.db.models.functions.text.Lower('name'), name='netbox_firmware_firmware_unique_name', violation_error_message="The Firmware 'Name' must be unique."),
        ),
    ]
