# Generated by Django 5.2.1 on 2025-09-10 11:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0207_remove_redundant_indexes'),
        ('netbox_firmware', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='firmware',
            options={'ordering': ('name', 'module_type')},
        ),
        migrations.RemoveConstraint(
            model_name='firmware',
            name='firmware_device_type_or_module_type_required',
        ),
        migrations.AlterUniqueTogether(
            name='firmware',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='firmware',
            name='device_type',
        ),
        migrations.AddField(
            model_name='firmware',
            name='device_type',
            field=models.ManyToManyField(blank=True, null=True, related_name='firmware', to='dcim.devicetype'),
        ),
    ]
