# Generated by Django 5.2.1 on 2025-09-23 08:19

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0207_remove_redundant_indexes'),
        ('netbox_firmware', '0006_alter_bios_options_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bios',
            name='manufacturer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='bios', to='dcim.manufacturer'),
        ),
    ]
